@@ -127,6 +127,8 @@ s48_value
 s48_external_call(s48_value sch_proc, s48_value proc_name,
 		  long nargs, char *char_argv)
 {
+  volatile char *gc_marker;		/* volatile to survive longjumps */
+  char *gc_marker_temp;			/* C wants it so */
   volatile char *gc_roots_marker;	/* volatile to survive longjumps */
   volatile s48_value name = proc_name;	/* volatile to survive longjumps */
   
@@ -144,7 +146,8 @@ s48_external_call(s48_value sch_proc, s48_value proc_name,
   S48_CHECK_VALUE(sch_proc);
   S48_CHECK_STRING(name);
 
-  gc_roots_marker = s48_set_gc_roots_baseB();
+  gc_roots_marker = s48_set_gc_roots_baseB(&gc_marker_temp);
+  gc_marker = gc_marker_temp;
 
   /* fprintf(stderr, "[external_call at depth %d]\n", depth); */
 
@@ -209,7 +212,7 @@ s48_external_call(s48_value sch_proc, s48_value proc_name,
 
     /* Raise an exception if the user neglected to pop off some gc roots. */
     
-    if (! s48_release_gc_roots_baseB((char *)gc_roots_marker)) {
+    if (! s48_release_gc_roots_baseB((char *)gc_roots_marker, (char *)gc_marker)) {
       s48_raise_scheme_exception(S48_EXCEPTION_GC_PROTECTION_MISMATCH, 0);
     }
     
@@ -242,7 +245,7 @@ s48_external_call(s48_value sch_proc, s48_value proc_name,
                        depth,
                       callback_depth());
      fprintf(stderr, "[throw unrolling to %ld]\n", gc_roots_marker); */
-    s48_release_gc_roots_baseB((char *)gc_roots_marker);
+    s48_release_gc_roots_baseB((char *)gc_roots_marker, (char *)gc_marker);
   }
 
   /* Check to see if a thread is waiting to return to the next block down. */
@@ -497,61 +500,130 @@ s48_raise_closed_channel_error() {
 void
 s48_raise_os_error(int the_errno) {
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 2,
-			     s48_enter_fixnum(the_errno),
+			     s48_enter_integer(the_errno),
 			     s48_enter_string(strerror(the_errno)));
 }
 
 void
 s48_raise_os_error_1(int the_errno, s48_value arg1) {
+
+  s48_value sch_errno = S48_UNSPECIFIC;
+  
+  S48_DECLARE_GC_PROTECT(2);
+  S48_GC_PROTECT_2 (arg1, sch_errno);
+  
+  sch_errno = s48_enter_integer(the_errno);
+
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 3,
-			     s48_enter_fixnum(the_errno),
+			     sch_errno,
 			     s48_enter_string(strerror(the_errno)),
 			     arg1);
+
+  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_2(int the_errno, s48_value arg1, s48_value arg2) {
+
+  s48_value sch_errno = S48_UNSPECIFIC;
+  
+  S48_DECLARE_GC_PROTECT(3);
+  S48_GC_PROTECT_3 (arg1, arg2, sch_errno);
+  
+  sch_errno = s48_enter_integer(the_errno);
+
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 4,
-			     s48_enter_fixnum(the_errno),
+                             sch_errno,
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2);
+  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_3(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3) {
+
+  s48_value sch_errno = S48_UNSPECIFIC;
+  
+  S48_DECLARE_GC_PROTECT(4);
+  S48_GC_PROTECT_4 (arg1, arg2, arg3, sch_errno);
+  
+  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 5,
-			     s48_enter_fixnum(the_errno),
+                             sch_errno,
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3);
+  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_4(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3, s48_value arg4) {
+
+  s48_value sch_errno = S48_UNSPECIFIC;
+  
+  S48_DECLARE_GC_PROTECT(5);
+  S48_GC_PROTECT_5 (arg1, arg2, arg3, arg4, sch_errno);
+  
+  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 6,
-			     s48_enter_fixnum(the_errno),
+                             sch_errno,
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3, arg4);
+  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_5(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3, s48_value arg4, s48_value arg5) {
+
+  s48_value sch_errno = S48_UNSPECIFIC;
+  
+  S48_DECLARE_GC_PROTECT(6);
+  S48_GC_PROTECT_6 (arg1, arg2, arg3, arg4, arg5, sch_errno);
+  
+  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 7,
-			     s48_enter_fixnum(the_errno),
+                             sch_errno,
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3, arg4, arg5);
+  S48_GC_UNPROTECT();
 }
 
 void
 s48_raise_os_error_6(int the_errno, s48_value arg1, s48_value arg2, 
 		     s48_value arg3, s48_value arg4, s48_value arg5,
 		     s48_value arg6) {
+
+  s48_value sch_errno = S48_UNSPECIFIC;
+  
+  S48_DECLARE_GC_PROTECT(7);
+  S48_GC_PROTECT_7 (arg1, arg2, arg3, arg4, arg5, arg6, sch_errno);
+  
+  sch_errno = s48_enter_integer(the_errno);
   s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 8,
-			     s48_enter_fixnum(the_errno),
+                             sch_errno,
 			     s48_enter_string(strerror(the_errno)),
 			     arg1, arg2, arg3, arg4, arg5, arg6);
+  S48_GC_UNPROTECT();
+}
+
+void
+s48_raise_os_error_7(int the_errno, s48_value arg1, s48_value arg2, 
+		     s48_value arg3, s48_value arg4, s48_value arg5,
+		     s48_value arg6, s48_value arg7) {
+
+  s48_value sch_errno = S48_UNSPECIFIC;
+  
+  S48_DECLARE_GC_PROTECT(8);
+  S48_GC_PROTECT_8 (arg1, arg2, arg3, arg4, arg5, arg6, arg7, sch_errno);
+  
+  sch_errno = s48_enter_integer(the_errno);
+  s48_raise_scheme_exception(S48_EXCEPTION_OS_ERROR, 9,
+                             sch_errno,
+			     s48_enter_string(strerror(the_errno)),
+			     arg1, arg2, arg3, arg4, arg5, arg6, arg7);
+  S48_GC_UNPROTECT();
 }
 
 void
@@ -752,7 +824,7 @@ s48_enter_unsigned_integer(unsigned long value)
 
     return s48_call_scheme(S48_SHARED_BINDING_REF(long_to_bignum_binding),
 			   3,
-			   S48_FALSE,   // this is ok
+			   S48_FALSE,   /* this is ok */
 			   S48_UNSAFE_ENTER_FIXNUM(value >> 16),
 			   S48_UNSAFE_ENTER_FIXNUM(value & 0xFFFF));
   }
@@ -924,6 +996,164 @@ s48_cons(s48_value v1, s48_value v2)
   return obj;
 }
 
+s48_value
+s48_list_1(s48_value v1)
+{
+  return (s48_cons (v1, S48_NULL));
+}
+
+s48_value
+s48_list_2(s48_value v1, s48_value v2)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_1 (v2);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_3(s48_value v1, s48_value v2, s48_value v3)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_2 (v2, v3);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_4(s48_value v1, s48_value v2, s48_value v3, s48_value v4)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_3 (v2, v3, v4);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_5(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_4 (v2, v3, v4, v5);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_6(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5,
+	   s48_value v6)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_5 (v2, v3, v4, v5, v6);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_7(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5,
+	   s48_value v6, s48_value v7)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_6 (v2, v3, v4, v5, v6, v7);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_8(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5,
+	   s48_value v6, s48_value v7, s48_value v8)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_7 (v2, v3, v4, v5, v6, v7, v8);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_9(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5,
+	   s48_value v6, s48_value v7, s48_value v8, s48_value v9)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_8 (v2, v3, v4, v5, v6, v7, v8, v9);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_10(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5,
+	    s48_value v6, s48_value v7, s48_value v8, s48_value v9, s48_value v10)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_9 (v2, v3, v4, v5, v6, v7, v8, v9, v10);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_11(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5,
+	    s48_value v6, s48_value v7, s48_value v8, s48_value v9, s48_value v10,
+	    s48_value v11)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_10 (v2, v3, v4, v5, v6, v7, v8, v9, v10, v11);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
+s48_value
+s48_list_12(s48_value v1, s48_value v2, s48_value v3, s48_value v4, s48_value v5,
+	    s48_value v6, s48_value v7, s48_value v8, s48_value v9, s48_value v10,
+	    s48_value v11, s48_value v12)
+{
+  s48_value list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(v1);
+  list = s48_list_11 (v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12);
+  list = s48_cons (v1, list);
+  S48_GC_UNPROTECT();
+  return list;
+}
+
 s48_value
 s48_make_weak_pointer(s48_value value)
 {
@@ -943,7 +1173,7 @@ s48_value
 s48_enter_substring(char *str, int length)
 {
   s48_value obj = s48_allocate_stob(S48_STOBTYPE_STRING, length + 1);
-  strncpy(S48_UNSAFE_EXTRACT_STRING(obj), str, length);
+  memcpy(S48_UNSAFE_EXTRACT_STRING(obj), str, length);
   *(S48_UNSAFE_EXTRACT_STRING(obj) + length) = '\0';
   return obj;
 }
@@ -1079,14 +1309,14 @@ s48_length(s48_value list)
   return S48_UNSAFE_ENTER_FIXNUM(i);
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// 
-//  Support for libscsh.a: add external initializers without the Makefile
-//
-////////////////////////////////////////////////////////////////////////////////
+/*
+** 
+**  Support for libscsh.a: add external initializers without the Makefile
+**
+*/
 
 struct simple_list{
-  void (*init)();          // pointer to init-function
+  void (*init)();          /* pointer to init-function */
   struct  simple_list* next;
 };
 
