@@ -15,8 +15,6 @@ char *s48_object_file;   /* specified via a command line argument */
 
 char *s48_reloc_file;    /* dynamic loading will set this */
 
-char *prog_name;
-
 void *heap, *stack;
 
 int s48_main (long heap_size, long stack_size,
@@ -30,7 +28,7 @@ int s48_main (long heap_size, long stack_size,
 }
     
 int
-internal_s48_main(long heap_size, long stack_size, char * _prog_name, 
+internal_s48_main(long heap_size, long stack_size,
 		  char* object_file, char *image_name, int argc, char** argv)
 {
   long return_value;
@@ -45,7 +43,6 @@ internal_s48_main(long heap_size, long stack_size, char * _prog_name,
   extern long i_count, *i_areas[], i_sizes[];
 #endif
 
-  prog_name = _prog_name;
 
   s48_object_file = object_file;
   s48_reloc_file = NULL;
