@@ -1,6 +1,7 @@
 /* Copyright (c) 1993-1999 by Richard Kelsey and Jonathan Rees.
    See file COPYING. */
 
+#include "sysdep.h"
 #include <signal.h>		/* for sigaction() (POSIX.1) */
 #include <stdlib.h>
 #include <unistd.h>
@@ -9,7 +10,6 @@
 #include <sys/time.h>
 #include <sys/times.h>
 #include <errno.h>              /* for errno, (POSIX?/ANSI) */
-#include "sysdep.h"
 #include "c-mods.h"
 #include "scheme48vm.h"
 #include "event.h"
@@ -17,17 +17,25 @@
 #include "../scsh/signals1.h"
 
 /* turning interrupts and I/O readiness into events */
+sigset_t full_sigset;
 
-#define block_interrupts()
-#define allow_interrupts()
+#define block_interrupts(){sigprocmask (SIG_BLOCK, &full_sigset, 0);}
+#define allow_interrupts(){sigprocmask (SIG_UNBLOCK, &full_sigset, 0);}
 
 
 static void	when_keyboard_interrupt();
 static void	when_alarm_interrupt();
 static void     when_sigpipe_interrupt();
 
-//JMG:
+/* JMG:*/
 static void     when_scsh_interrupt();
+/* JMG: for scsh */
+#define INTERRUPT_QUEUE_LENGTH 32
+
+static int      interrupt_queue [INTERRUPT_QUEUE_LENGTH];
+static int	next_interrupt = 0;
+static int s48_os_signal_pending(void);
+static bool s48_os_signal_happend(void);
 
 
 bool		s48_setcatcher(int signum, void (*catcher)(int));
@@ -46,7 +54,9 @@ s48_sysdep_init(void)
     exit(1);
   }
 
-  //JMG: for scsh
+  sigfillset (&full_sigset);
+      
+  /* JMG: for scsh */
   s48_setcatcher(SIGCHLD,   when_scsh_interrupt);
   s48_setcatcher(SIGCONT,   when_scsh_interrupt);
   s48_setcatcher(SIGHUP,    when_scsh_interrupt);
@@ -101,9 +111,9 @@ s48_setcatcher(int signum, void (*catcher)(int))
     fprintf(stderr, "Failed to get sigaction for signal %d\n", signum);
     exit(1);
   }
-  //  JMG: what's the point of not setting the handler in this case?
-  //  if (sa.sa_handler == SIG_IGN)
-  //  return (TRUE);
+  /*  JMG: what's the point of not setting the handler in this case?
+      if (sa.sa_handler == SIG_IGN)
+      return (TRUE);*/
   sa.sa_handler = catcher;
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
@@ -276,7 +286,7 @@ s48_stop_alarm_interrupts(void)
  *                  (queue-ready-ports)
  *                  (set! *poll-time* (+ *time* *poll-interval*))))
  *           (cond ((not (queue-empty? ready-ports))
- *                  (values (enum event-type i/o-completion)
+ *                  (values (enum event-type i/o-{read/write}-completion)
  *                          (dequeue! ready-ports)))
  *                 ((>= *current_time* *alarm-time*)
  *                  (set! *alarm-time* max-integer)
@@ -291,9 +301,20 @@ s48_stop_alarm_interrupts(void)
  *                  (values (enum event-type no-event) #f))))))
  */
 
-static bool	there_are_ready_ports(void);
-static int	next_ready_port(void);
-static int	queue_ready_ports(bool wait, long seconds, long ticks);
+#define FD_QUIESCENT 0			/* idle */
+#define FD_READY     1			/* I/O ready to be performed */
+#define FD_PENDING   2			/* waiting */
+
+typedef struct fd_struct {
+ int	fd,			/* file descriptor */
+	status;			/* one of the FD_* constants */
+ bool	is_input;		/* iff input */
+ struct fd_struct	*next;	/* next on same queue */
+} fd_struct;
+
+static bool	 there_are_ready_ports(void);
+static fd_struct *next_ready_fd_struct(void);
+static int	 queue_ready_ports(bool wait, long seconds, long ticks);
 
 int
 s48_get_next_event(long *ready_fd, long *status)
@@ -303,6 +324,8 @@ s48_get_next_event(long *ready_fd, long *status)
   */
 
   int io_poll_status;
+  fd_struct *f;
+
   /*
     fprintf(stderr, "[poll at %d (waiting for %d)]\n", s48_current_time, alarm_time);
     */
@@ -323,21 +346,25 @@ s48_get_next_event(long *ready_fd, long *status)
     }
   }
   if (there_are_ready_ports()) {
-    *ready_fd = next_ready_port();
+    f = next_ready_fd_struct();
+    *ready_fd = f->fd;
     *status = 0;   /* chars read or written */
     /* fprintf(stderr, "[i/o completion]\n"); */
-    return (IO_COMPLETION_EVENT);
+    if (f->is_input)
+      return (IO_READ_COMPLETION_EVENT);
+    else
+      return (IO_WRITE_COMPLETION_EVENT);
   }
   if (alarm_time != -1 && s48_current_time >= alarm_time) {
     alarm_time = -1;
     /* fprintf(stderr, "[alarm]\n"); */
     return (ALARM_EVENT);
   }
-  //JMG: scsh should handle this
+  block_interrupts();
+  /* JMG: scsh should handle this */
   if (s48_os_signal_pending())
     return (OS_SIGNAL_EVENT);
     
-  block_interrupts();
   if ((keyboard_interrupt_count == 0)
       &&  (alarm_time == -1 || s48_current_time < alarm_time)
       &&  (poll_time == -1 || s48_current_time < poll_time))
@@ -353,17 +380,6 @@ s48_get_next_event(long *ready_fd, long *status)
  * the pending ports and move any that are ready onto the other queue and
  * signal an event.
  */
-#define FD_QUIESCENT 0			/* idle */
-#define FD_READY     1			/* I/O ready to be performed */
-#define FD_PENDING   2			/* waiting */
-
-typedef struct fd_struct {
- int	fd,			/* file descriptor */
-	status;			/* one of the FD_* constants */
- bool	is_input;		/* iff input */
- struct fd_struct	*next;	/* next on same queue */
-} fd_struct;
-
 
 /*
  * A queue of fd_structs is empty iff the first field is NULL.  In
@@ -448,14 +464,14 @@ there_are_ready_ports(void)
 }
 
 
-static int
-next_ready_port(void)
+static fd_struct *
+next_ready_fd_struct(void)
 {
   fd_struct	*p;
 
   p = rmque(&ready.first, &ready);
   p->status = FD_QUIESCENT;
-  return (p->fd);
+  return (p);
 }
 
 
@@ -562,6 +578,8 @@ s48_wait_for_event(long max_wait, bool is_minutes)
   }
   if (keyboard_interrupt_count > 0)
     status = NO_ERRORS;
+  else if (s48_os_signal_happend ())
+    status = NO_ERRORS;
   else {
     status = queue_ready_ports(TRUE, seconds, ticks);
     if (there_are_ready_ports())
@@ -606,8 +624,9 @@ queue_ready_ports(bool wait, long seconds, long ticks)
   }
   tvp = &tv;
   if (wait)
-    if (seconds == -1)
+    if (seconds == -1){
       tvp = NULL;
+    }
     else {
       tv.tv_sec = seconds;
       tv.tv_usec = ticks * (1000000 / TICKS_PER_SECOND);
@@ -615,6 +634,9 @@ queue_ready_ports(bool wait, long seconds, long ticks)
   else
     timerclear(&tv);
   while(TRUE) {
+    if ((keyboard_interrupt_count > 0) || s48_os_signal_happend ())
+      return NO_ERRORS;
+    /* time gap */
     left = select(limfd, &reads, &writes, &alls, tvp);
     if (left > 0) {
       fdpp = &pending.first;
@@ -642,13 +664,25 @@ queue_ready_ports(bool wait, long seconds, long ticks)
   }
 }
 
+/*
+ * Adds `signum' to the queue of received signals.
+ */
 
-//JMG: for scsh
-static long interrupt_count[32];
+static void
+queue_interrupt(int signum)
+{
+  if (next_interrupt == INTERRUPT_QUEUE_LENGTH){
+    perror("Interrupt queue overflow -- report to Scheme 48 maintainers.");
+    exit(-1); 
+  }
+  interrupt_queue[next_interrupt] = signum;
+  next_interrupt++;
+}
 
+/* JMG: for scsh */
 static void when_scsh_interrupt(int signo)
 {
-  interrupt_count[sig2int[signo]] +=1;
+  queue_interrupt(sig2int[signo]);
   NOTE_EVENT;
   return;
 }
@@ -676,18 +710,34 @@ static void when_scsh_interrupt(int signo)
  * reenabled when the handler returns (or if done by hand).
  */
 
+/*
+ * Returns TRUE if there is a signal to be delivered up to Scheme.
+ * Needs no be called with interrupts blocked.
+ */
+
 int
 s48_os_signal_pending(void) {
   int i;
-  for (i = 0; i < max_sig; i++){
-    if (interrupt_count[i] > 0){
-      block_interrupts();
-      --interrupt_count[i];
-      allow_interrupts();
-      s48_set_os_signal(S48_UNSAFE_ENTER_FIXNUM(i),
-			S48_UNSAFE_ENTER_FIXNUM(0));
-      return TRUE;
-    }
-  }
-  return FALSE;
+  s48_value interrupt_list = S48_NULL;
+  block_interrupts();
+
+  if (next_interrupt == 0) {
+    allow_interrupts();
+    return FALSE; }
+  else {
+    /* turn the queue into a scheme list and preserve the order */
+    for (i = next_interrupt; i > 0 ; i--)
+      interrupt_list = s48_cons (s48_enter_fixnum (interrupt_queue  [i - 1]),
+				 interrupt_list);
+    s48_set_os_signals(interrupt_list);
+		      
+    next_interrupt = 0;
+    allow_interrupts();
+    return TRUE; }
 }
+
+bool 
+s48_os_signal_happend(void) {
+  return (next_interrupt != 0);
+}
+
