@@ -186,12 +186,12 @@
 ;;; Emit C code to copy a C string into its carrier.
 (define (str-and-len->carrier carrier str)
   (format #f
-	  "SetAlienVal(S48_CAR(~a),(long) ~a); S48_SET_CDR(~a,strlen_or_false(~a));//str-and-len"
+	  "SetAlienVal(S48_CAR(~a),(long) ~a); S48_SET_CDR(~a,strlen_or_false(~a));"
 	  carrier str carrier str))
 
 ;;; Carrier and assignment-generator for alien values:
 (define (simple-assign carrier val)
-  (format #f "SetAlienVal(~a,(long) ~a);//simple-assign" carrier val)) ;;JMG: untested
+  (format #f "SetAlienVal(~a,(long) ~a);" carrier val)) ;;JMG: untested
 
 ;;; Note: When MAKE-CARRIER and S-CVTR fields are taken from this table,
 ;;; they are symbols that are syntactically closed in the macro expander's
