@@ -223,20 +223,6 @@
             (apply define-data-struct-primitives stuff))
           stob-data)
 
-; For flat environments
-
-(let ((:value (sexp->type ':value #t))
-      (:vector (sexp->type ':vector #t)))
-  (define-simple-primitive 'make-cell
-    (proc (:value) :vector)
-    (instruction (enum op make-stored-object) 1 (enum stob vector)))
-  (define-simple-primitive 'cell-ref
-    (proc (:vector) :value)
-    (instruction (enum op stored-object-ref) (enum stob vector) 0))
-  (define-simple-primitive 'cell-set!
-    (proc (:vector :value) unspecific-type)
-    (instruction (enum op stored-object-set!) (enum stob vector) 0)))
-
 ; Define primitives for vector-like stored objects.
 
 (define (define-vector-primitives name element-type make length ref set!)
