@@ -12,16 +12,23 @@
 
 (define (read-forms pathname package)
   (let* ((filename (namestring pathname #f *scheme-file-type*))
-	 (truename (translate filename)))
-    (call-with-input-file truename
-      (lambda (port)
-	((fluid $note-file-package) filename package)
-	(let ((o-port (current-noise-port)))
-	  (display truename o-port)
-	  (force-output o-port)
-	  (really-read-forms port))))))
+	 (truename (translate filename))
+	 (port (open-input-file truename)))
+    (dynamic-wind
+     (lambda ()
+       (if (not port)
+	   (error "attempt to throw back into a file read"))) ; message needs work
+     (lambda ()
+       ((fluid $note-file-package) filename package)
+       (let ((o-port (current-noise-port)))
+	 (display truename o-port)
+	 (force-output o-port)
+	 (read-forms-from-port port)))
+     (lambda ()
+       (close-input-port port)
+       (set! port #f)))))
 
-(define (really-read-forms port)	    
+(define (read-forms-from-port port)	    
   (let loop ((forms '()))
     (let ((form (read port)))
       (if (eof-object? form)
