@@ -54,6 +54,7 @@
 
 (define-interface primitives-interface
   (export add-finalizer!
+	  add-pending-channel
 	  call-external-value
 	  checked-record-ref
 	  checked-record-set!
@@ -112,6 +113,12 @@
 	  ((bitwise-and bitwise-ior bitwise-xor)
 	   (proc (&rest :exact-integer) :exact-integer))))
 
+(define-interface cells-interface
+  (export cell?
+	  make-cell
+	  cell-ref
+	  cell-set!))
+
 (define-interface locations-interface
   (export location?
 	  location-defined?
@@ -134,7 +141,9 @@
 	  byte-vector-length
 	  byte-vector-ref
 	  byte-vector-set!
-	  make-byte-vector))
+	  make-byte-vector
+	  
+	  byte-vector))
 
 ; Same again, but with old names for compatibility.
 
@@ -191,6 +200,7 @@
   (export lock?
 	  make-lock
 	  obtain-lock
+	  obtain-lock-multiple
 	  maybe-obtain-lock
 	  release-lock
 	  lock-owner-uid))    ;really should be internal
@@ -299,7 +309,9 @@
 	  reduce
 	  sublist
 	  insert
-	  unspecific))
+	  unspecific
+	  
+	  (mvlet :syntax)))
 
 ; Level 2 consists of harder things built on level 1.
 
@@ -386,6 +398,10 @@
 	  let-fluid
 	  let-fluids
 	  fluid
+	  
+	  fluid-cell-ref
+	  fluid-cell-set!
+
 	  set-fluid!))
 
 (define-interface fluids-internal-interface
@@ -520,7 +536,6 @@
 	  initialize-channel-i/o!       ;scheduler
 	  waiting-for-i/o?              ;scheduler
 
-	  steal-channel-port!		;command
 	  steal-channel!                ;JMG: 3 For scsh.
 	  steal-port! 
 	  channel-cell-ref
@@ -532,6 +547,7 @@
 	  output-channel+closer->port	;big/socket.scm
 	  
 	  ; call WAIT-FOR-CHANNEL with interrupts disabled
+	  wait-for-channels
 	  wait-for-channel		;big/socket.scm
 
 	  port->channel			;posix
@@ -542,7 +558,7 @@
   (export thread?
 	  thread-name thread-uid        ;for debugging
 
-	  spawn
+	  spawn narrow
 	  relinquish-timeslice
 	  sleep
 	  terminate-current-thread))
@@ -557,12 +573,10 @@
 
 	  current-thread
 
-	  make-thread-queue
-	  thread-queue-empty?
 	  enqueue-thread!
-	  multiple-enqueue-thread!
-	  dequeue-thread!
-	  remove-thread-from-queues!
+	  thread-queue-empty?
+	  maybe-dequeue-thread!
+	  block-on-queue
 
 	  event-pending?
 	  get-next-event!
@@ -574,13 +588,16 @@
 	  
 	  block
 	  make-ready
+	  set-thread-cell! clear-thread-cell!
 	  spawn-on-scheduler spawn-on-root
 	  wait
 	  upcall propogate-upcall
 	  interrupt-thread
 	  kill-thread!
+	  terminate-thread!
 
 	  wake-some-threads
+	  register-dozer
 
 	  all-threads                   ; for command-levels
 
@@ -590,6 +607,7 @@
   (export run-threads
 	  run-threads-with-housekeeper
 	  round-robin-event-handler
+	  handle-narrow-event
  
 	  make-counter            ; for thread counts
 	  counter-value
@@ -597,8 +615,9 @@
           decrement-counter!))
 
 (define-interface queues-interface
-  (export make-queue enqueue! dequeue! queue-empty?
-	  queue? queue->list queue-length delete-from-queue!))
+  (export make-queue enqueue! dequeue! maybe-dequeue! queue-empty?
+	  queue? queue->list queue-front queue-length
+	  delete-from-queue! on-queue?))
 
 (define-interface exceptions-interface
   (export define-exception-handler
@@ -636,7 +655,7 @@
 
 (define-interface rts-sigevents-internal-interface
   (export waiting-for-sigevent?
-	  initialize-sigevents!))
+	  with-sigevents))
 
 (define-interface writing-interface
   (export write
@@ -990,10 +1009,12 @@
 ; Interfaces.
 
 (define-interface interfaces-interface
-  (export make-compound-interface
-	  make-simple-interface
+  (export make-simple-interface
+	  make-compound-interface
+	  make-modified-interface
 	  note-reference-to-interface!
 	  interface-ref
+	  interface-member?
 	  interface?
 	  interface-clients
 	  for-each-declaration
@@ -1005,6 +1026,7 @@
   (export make-package
 	  make-simple-package		;start.scm
 	  make-structure
+	  make-modified-structure
 	  package-define!
 	  package-lookup
 	  package?			;command.scm
@@ -1123,6 +1145,7 @@
 	    define-reflective-tower-maker
 	    export-reflective-tower-maker
 	    compound-interface
+	    modify subset with-prefix
 	    export
 	    structure structures let	; New
 	    begin			; mostly for macros
@@ -1158,6 +1181,7 @@
 
 (define-interface evaluation-interface
   (export eval load load-into eval-from-file
+          load-port
 	  ; eval-scanned-forms
 	  ))
 
