@@ -13,7 +13,9 @@
     (initialize-rts in out error
      (lambda ()
        (initialize-records! records)
-       (entry-point (vector->list resume-arg))))))
+       (with-sigevents
+	(lambda ()
+	  (entry-point (vector->list resume-arg))))))))
        
 (define (initialize-rts in out error thunk)
   (initialize-session-data!)
@@ -30,7 +32,6 @@
 	     (lambda ()
 	       (with-threads
 		(lambda ()
-		  (initialize-sigevents!)
 		  (root-scheduler thunk
 				  200           ; thread quantum, in msec
 				  300)))))))))) ; port-flushing quantum
