@@ -12,7 +12,7 @@
 ;   saved interrupt mask
 ;   scheduler, which is the thread that RUNs this one
 ;   remaining time in clock ticks ('waiting = waiting for events)
-;   queue that is holding this thread, if any
+;   cell that is holding this thread, if any
 ;   arguments waiting to be passed to the thread when it is next run
 ;   dynamic environment
 ;   dynamic point
@@ -41,7 +41,7 @@
 (define-record-type thread :thread
   (really-make-thread dynamic-env dynamic-point cell-env
 		      continuation scheduler
-		      queues arguments
+		      cell arguments
 		      events current-task uid name)
   thread?
   (dynamic-env    thread-dynamic-env)   ;Must be first!  (See fluid.scm)
@@ -49,7 +49,7 @@
   					;Must be second!  (See fluid.scm)
   (cell-env       thread-cell-env)      ;Must be fourth! (See thread-env.scm)
   (continuation   thread-continuation set-thread-continuation!)
-  (queues         thread-queues set-thread-queues!)
+  (cell           thread-cell set-thread-cell!)
   (arguments      thread-arguments set-thread-arguments!)
   (time           thread-time set-thread-time!)
   (scheduler      thread-scheduler set-thread-scheduler!)
@@ -77,7 +77,7 @@
 				    (thunk->continuation
 				     (thread-top-level thunk))
 				    (current-thread) ; scheduler
-				    #f               ; queue
+				    #f               ; cell
 				    '()              ; arguments
 				    #f               ; events
 				    #f               ; current-task
@@ -180,32 +180,31 @@
 ; Rename the queue operations as thread-specific ones (both for clarity
 ; and because we will want to use priority queues in the future).
 
-(define make-thread-queue make-queue)
-(define thread-queue-empty? queue-empty?)
-
 (define (enqueue-thread! queue thread)
-  (if (thread-queues thread)
-      (error "enqueued thread being added to another queue" thread queue))
-  (set-thread-queues! thread (list queue))
-  (enqueue! queue thread))
+  (let ((cell (make-cell thread)))
+    (enqueue! queue cell)
+    (set-thread-cell! thread cell)))
 
-(define (multiple-enqueue-thread! queues thread)
-  (if (thread-queues thread)
-      (error "enqueued thread being added to another queue" thread queues))
-  (set-thread-queues! thread queues)
-  (for-each (lambda (q) (enqueue! q thread)) queues))
+(define (maybe-dequeue-thread! queue)
+  (let loop ()
+    (let ((cell (maybe-dequeue! queue)))
+      (if cell
+	  (or (cell-ref cell)
+	      (loop))
+	  #f))))
 
-(define (dequeue-thread! queue)
-  (let ((thread (dequeue! queue)))
-    (for-each (lambda (q) (delete-from-queue! q thread)) (thread-queues thread))
-    (set-thread-queues! thread #f)
-    thread))
+; Look for a non-empty cell.
+
+(define (thread-queue-empty? queue)
+  (let loop ()
+    (cond ((queue-empty? queue)
+	   #t)
+	  ((cell-ref (queue-front queue))
+	   #f)
+	  (else
+	   (dequeue! queue)
+	   (loop)))))
 
-(define (remove-thread-from-queues! thread)
-  (if (thread-queues thread)
-      (begin
-	(for-each (lambda (q) (delete-from-queue! q thread)) (thread-queues thread))
-	(set-thread-queues! thread #f))))
 ;----------------
 
 (define current-thread (structure-ref primitives current-thread))
@@ -226,6 +225,7 @@
   ;; asynchronous events
    runnable       ; <thread> <args>     <thread> is now runnable
    spawned        ; <thunk> <id> ...    spawn <thunk> as a new thread
+   narrowed       ; <thunk> <id> ...    narrow to <thunk> as a new thread
    interrupt      ; <type> . <stuff>    an interrupt has occured
    deadlock       ;                     no one can run
  
@@ -256,13 +256,9 @@
 	  ((not (eq? (thread-scheduler thread) scheduler))
 	   (enable-interrupts!)
 	   (error "thread run by wrong scheduler" thread scheduler))
-	  ((thread-queues thread)
+	  ((thread-cell thread)
 	   (enable-interrupts!)
-	   (apply debug-message (list "thread run while still on a queue " 
-				      (thread-uid thread)
-				      (thread-name thread)
-				      (thread-queues thread)))
-	   (error "thread run while still on a queue" thread))
+	   (error "thread run while still blocked" thread))
 	  ((and (thread-current-task thread)
 		(not (null? (thread-arguments thread))))
 	   (enable-interrupts!)
@@ -442,6 +438,12 @@
 (define (relinquish-timeslice)
   (suspend (enum event-type out-of-time) '()))
 
+; Utility procedure for the common case of blocking on a queue.
+
+(define (block-on-queue queue)
+  (enqueue-thread! queue (current-thread))
+  (block))
+
 ; Send the upcall to the current scheduler and check the return value(s)
 ; to see if it was handled properly.
 
@@ -464,6 +466,13 @@
 		    (lambda ignored
 		      (exit (enum event-type killed) '()))))
 
+; Also ends the thread, but lets it run any pending dynamic-winds.
+
+(define (terminate-thread! thread)
+  (let ((interrupts (set-enabled-interrupts! no-interrupts)))
+    (clear-thread-cell! thread)
+    (interrupt-thread thread (lambda args (terminate-current-thread)))))
+
 ;----------------
 ; Make THREAD execute PROC the next time it is run.  The thread's own
 ; continuation is passed whatever PROC returns.
@@ -586,13 +595,16 @@
 		  thunk
 		  (if (null? id) #f (car id))))
 
+(define (narrow thunk . id)
+  (schedule-event (thread-scheduler (current-thread))
+		  (enum event-type narrowed)
+		  thunk
+		  (if (null? id) #f (car id))))
+
 ; Enqueue a RUNNABLE for THREAD's scheduler.
 
 (define (make-ready thread . args)
-  (if (thread-queues thread)
-      (error "trying to schedule a queued thread" thread))
-;  (if (not (null? (thread-arguments thread)))
-;      (error "trying to replace thread arguments"))
+  (clear-thread-cell! thread)
   (set-thread-arguments! thread args)
   (if (thread-scheduler thread)
       (schedule-event (thread-scheduler thread)
@@ -600,6 +612,13 @@
 		      thread)
       (error "MAKE-READY thread has no scheduler" thread)))
 
+(define (clear-thread-cell! thread)
+  (let ((cell (thread-cell thread)))
+    (if cell
+	(begin
+	  (set-thread-cell! thread #f)
+	  (cell-set! cell #f)))))
+
 ;----------------
 
 (define (schedule-interrupt! time)
