@@ -28,7 +28,6 @@
 ; Interrupts
 
 (define *enabled-interrupts*)	; bitmask of enabled interrupts
-(define *pending-interrupts*)	; bitmask of pending interrupts
 (define s48-*pending-interrupt?*) ; true if an interrupt is pending
 
 (define *interrupted-template*) ; template in place when the most recent
@@ -60,7 +59,7 @@
   (set! *finalizer-alist*    null)
   (set! *finalize-these*     null)
 
-  (set! *pending-interrupts* 0)
+  (pending-interrupts-clear!)
   (set! s48-*pending-interrupt?* #f)
   (set! *interrupted-template* false)
   unspecific-value)
@@ -81,8 +80,7 @@
     (set! *interrupt-template*   (s48-trace-value *interrupt-template*))
     (set! *interrupted-template* (s48-trace-value *interrupted-template*))
     (set! *finalize-these*       (s48-trace-value *finalize-these*))
-    (set! *os-signal-type*       (s48-trace-value *os-signal-type*))
-    (set! *os-signal-argument*   (s48-trace-value *os-signal-argument*))
+    (set! *os-signal-list*       (s48-trace-value *os-signal-list*))
     (trace-finalizer-alist!)
     
     ; These could be moved to the appropriate modules.
@@ -639,6 +637,7 @@
 	  ((and (false? cont)
 		(fixnum? *val*))			  ; VM returns here
 	   (set! s48-*callback-return-stack-block* false) ; not from a callback
+	   (reset-stack-pointer false)                    ; for libscsh
 	   (extract-fixnum *val*))
 	  (else
 	   (set-current-continuation! false)
