@@ -11,13 +11,15 @@
 
 #include "cstuff.h"
 
+extern int errno;
+
 /* stub used to return TRUE when scheme checks for Berkeley dbm */
 int db_check()
 {
   return 1;
   }
 
-scheme_value db_open_default(char *file, int flags, int mode, DB **db_ptr)
+s48_value db_open_default(char *file, int flags, int mode, DB **db_ptr)
 {
     *db_ptr = dbopen(file, flags, mode, DB_BTREE, NULL);
     if(*db_ptr == NULL)
@@ -30,7 +32,7 @@ scheme_value db_open_default(char *file, int flags, int mode, DB **db_ptr)
     Note: if pass_info is set to zero, all subsequent arguments are
           ignored, otherwise they are loaded into a BTREEINFO structure
           and passed to the open routine */
-scheme_value db_open_btree (char *file, int flags, int mode, int pass_info,
+s48_value db_open_btree (char *file, int flags, int mode, int pass_info,
 		     int access_flags, int cachesize, int maxkeypage,
 		     int minkeypage, int psize, int lorder, DB **db_ptr)
 {
@@ -56,7 +58,7 @@ scheme_value db_open_btree (char *file, int flags, int mode, int pass_info,
     }
 
 /*  Open a hash type database (same use of pass_info as in btree) */
-scheme_value db_open_hash (char *file, int flags, int mode, int pass_info,
+s48_value db_open_hash (char *file, int flags, int mode, int pass_info,
 		     int bsize, int ffactor, int nelem, int cachesize,
 		     int lorder, DB **db_ptr)
 {
@@ -80,7 +82,7 @@ scheme_value db_open_hash (char *file, int flags, int mode, int pass_info,
     }
 
 /*  Open a recno type database (with same use of pass_info again) */
-scheme_value db_open_recno (char *file, int flags, int mode, int pass_info,
+s48_value db_open_recno (char *file, int flags, int mode, int pass_info,
 		     int access_flags, int cachesize, int psize, int lorder,
 		     int reclen, char bval, char *bfname, DB **db_ptr)
 {
