@@ -1,5 +1,5 @@
 ;;; Scsh routines for analysing exit codes returned by WAIT.
-;;; Copyright (c) 1994 by Olin Shivers. See file COPYING.
+;;; Copyright (c) 1994 by Olin Shivers.
 ;;;
 ;;; To port these to a new OS, consult /usr/include/sys/wait.h, 
 ;;; and check the WIFEXITED, WEXITSTATUS, WIFSTOPPED, WSTOPSIG, 
@@ -31,7 +31,7 @@
 (define (status:term-sig status)
   (let ((termsig (bitwise-and status #x7F)))
     (and (not (zero? termsig))				; Didn't exit.
-	 (not (= #x7F))					; Not suspended.
+	 (not (= termsig #x7F))				; Not suspended.
 	 termsig)))
 
 
