@@ -32,7 +32,7 @@
 (define (status:term-sig status)
   (let ((termsig (bitwise-and status #x7F)))
     (and (not (zero? termsig))				; Didn't exit.
-	 (not (= #x7F))					; Not suspended.
+	 (not (= termsig #x7F))				; Not suspended.
 	 termsig)))
 
 
