@@ -1,4 +1,4 @@
-;;; Copyright (c) 1994 by Olin Shivers. See file COPYING.
+;;; Copyright (c) 1994 by Olin Shivers.
 
 ;;; Handy for defining random flags and constants.
 ;;; (define-enum-constant "errno" intr 9) => (define errno/intr 9)
@@ -33,3 +33,18 @@
   (syntax-rules ()
     ((define-enum-constants set (elt val) ...)
      (begin (define-enum-constant set elt val) ...))))
+
+(define-syntax define-enum-constants-from-x
+  (syntax-rules 
+   ()
+   ((define-enum-constants-from-zero x set ())
+    (begin))
+   ((define-enum-constants-from-zero x set (elt1 elt2 ...))
+    (begin (define-enum-constant set elt1 x)
+	   (define-enum-constants-from-x (+ x 1) set (elt2 ...))))))
+
+(define-syntax define-enum-constants-from-zero
+  (syntax-rules 
+   ()
+   ((define-enum-constants-from-zero set (elt ...))
+    (define-enum-constants-from-x 0 set (elt ...)))))
