@@ -72,7 +72,7 @@ s48_value read_delim(const char *delims, char *buf,
 
 	else if( retval == -1 ) {       /* Terminal case: error. */
 	    *nread =  cptr - buf - start;
-	    return s48_enter_fixnum(errno);
+	    return s48_enter_integer(errno);
 	    }	  
 
 	else if( delims[c] ) {		/* Terminal case: delimiter char. */
@@ -115,7 +115,7 @@ s48_value skip_chars(const char *skipchars, int fd,  int *nread)
 
 	if( retval == -1 ) {		/* Terminal case: error. */
 	    *nread = nr;
-	    return s48_enter_fixnum(errno);
+	    return s48_enter_integer(errno);
 	    }
 
 	else if( !skipchars[c] ) {	/* Terminal case: non-skip char. */
