@@ -5,7 +5,8 @@
   (export open/sync))
 
 (define-interface hpux-errno-extras-interface
-  (export errno/wouldblock))
+  (export errno/wouldblock
+	  errno/connaborted))
 
 (define-interface hpux-signals-extras-interface
   (export))
