@@ -11,13 +11,15 @@
 
 #include "cstuff.h"
 
+extern int errno;
+
 /* stub used to return FALSE when scheme checks for Berkeley dbm */
 int db_check()
 {
   return 0;
   }
 
-scheme_value db_open_default (char *file, int flags, int mode, DBM **db_ptr)
+s48_value db_open_default (char *file, int flags, int mode, DBM **db_ptr)
 {
     *db_ptr = dbm_open(file, flags, mode);
     if(*db_ptr == NULL)
