@@ -42,11 +42,23 @@
 (make-reinitializer (lambda ()
 		      (set! process-table (make-integer-table))))
 
+(define process-table-lock (make-lock))
 (define (process-table-ref n)
-  (weak-table-ref process-table n))
+  (with-lock process-table-lock
+	     (lambda ()
+	       (weak-table-ref process-table n))))
 
 (define (process-table-set! n val)
-  (weak-table-set! process-table n val))
+  (with-lock process-table-lock
+	     (lambda ()
+	       (weak-table-set! process-table n val))))
+
+(define (process-table-delete-procobj! procobj)
+  (with-lock process-table-lock
+	     (lambda ()
+	       (if (eq? (weak-table-ref process-table (proc:pid procobj))
+			procobj)
+		   (weak-table-set! process-table (proc:pid procobj) #f)))))
 
 (define (maybe-pid->proc pid)
   (process-table-ref pid))
@@ -150,9 +162,9 @@
 	   (let lp ((event (most-recent-sigevent)))
 	     (let ((next-event (next-sigevent event interrupt/post-gc)))
 	       (*post/gc-handler*)
-	       (lp next-event))))
-	 '*post/gc-handler*-thread))
-  
+ 	       (lp next-event))))
+ 	 '*post/gc-handler*-thread))
+
 (define set-post/gc-handler! start-set-post/gc-handler!)
 
 
@@ -169,6 +181,7 @@
 	 (*sigchld-handler*)
 	 (lp next-event))))
    thunk
+   (structure-ref threads-internal spawn-on-root)
    'auto-reaping))
 
 ;;; This list contains pids whose proc-obj were gc'd before they died
@@ -184,7 +197,7 @@
 
 (define (need-reaping-remove! pid)
   (obtain-lock need-reaping-lock)
-  (set! need-reaping (del pid need-reaping))
+  (set! need-reaping (delete pid need-reaping))
   (release-lock need-reaping-lock))
 
 (define (reap-need-reaping)
@@ -195,10 +208,13 @@
 ;;; reap this special pid
 ;;; return status or #f 
 (define (reap-pid pid)
-  (let ((status (atomic-wait pid wait/poll)))
-    (if status 
-	(waited-by-reap pid status))
-    status))
+  (with-lock 
+   wait-lock
+   (lambda ()
+     (let ((status (atomic-wait pid wait/poll)))
+       (if status 
+	   (waited-by-reap pid status))
+       status))))
 	
 ;;; Handler for SIGCHLD according policy
 (define (late-sigchld-handler) #f)
@@ -212,6 +228,7 @@
 ;;; Finalizer for procobjs
 ;;; 
 (define (procobj-finalizer procobj)
+  (process-table-delete-procobj! procobj)
   (if (not (proc:finished? procobj))
       (need-reaping-add! (proc:pid procobj))))
 
@@ -222,14 +239,18 @@
 
 (define (reap-zombies)
   (let lp ()
-    (receive (pid status) 
+    (obtain-lock wait-lock)
+    (receive (pid status)
       (%wait-any (bitwise-ior wait/poll wait/stopped-children))
-      (if pid
+      (if (and pid (not (status:stop-sig status)))
 	  (begin (waited-by-reap pid status)
+		 (release-lock wait-lock)
 ;		 (format (current-error-port)
 ;			 "Reaping ~d[~d]~%" pid status)
 		 (lp))
-	  status))))
+	  (begin 
+	    (release-lock wait-lock)
+	    status)))))
 
 
 
@@ -260,7 +281,7 @@
 
 
 ;;; With this lock, we ensure that only one thread may call
-;;; really-wait for a given pid
+;;; really-wait for a given pid and manipulates the associated process object
 
 (define wait-lock (make-lock))
 
@@ -272,26 +293,29 @@
 		status)))
     ;; save the event before we check for finished
     (let ((pre-event (most-recent-sigevent)))
-      (cond ((atomic-wait proc (bitwise-ior flags wait/poll)) => win)
-	    
-	    ((zero? (bitwise-and flags wait/poll))
-	     ;; we have to block and hence use the event system
-	     (let lp ((pre-event pre-event))
-	       (cond ((atomic-wait proc (bitwise-ior flags wait/poll)) 
-		      => win)
-		     (else 
-		      (lp (next-sigevent pre-event interrupt/chld))))))
-	    (else #f)))))
+      (with-lock 
+       wait-lock
+       (lambda () 
+	 (cond ((atomic-wait proc (bitwise-ior flags wait/poll)) => win)
+	       
+	       ((zero? (bitwise-and flags wait/poll))
+		;; we have to block and hence use the event system
+		(let lp ((pre-event pre-event))
+		  (cond ((atomic-wait proc (bitwise-ior flags wait/poll)) 
+			 => win)
+			(else
+			 (release-lock wait-lock)
+			 (let ((next-event (next-sigevent pre-event interrupt/chld)))
+			   (obtain-lock wait-lock)
+			   (lp next-event))))))
+	       (else #f)))))))
 		
 
 ;;; -> process-object proc status/#f
 (define (atomic-wait proc flags)
-  (with-lock
-   wait-lock
-   (lambda ()
-     (cond ((proc:finished? proc)
-	    (placeholder-value (proc:status proc)))
-	   (else (really-wait (proc:pid proc) (bitwise-ior flags wait/poll)))))))
+  (cond ((proc:finished? proc)
+	 (placeholder-value (proc:status proc)))
+	(else (really-wait (proc:pid proc) (bitwise-ior flags wait/poll)))))
 	   
 ;;; This one is used, to wait on a positive pid
 ;;; We NEVER do a blocking wait syscall 
@@ -315,13 +339,16 @@
   (cond ((maybe-pid->proc pid) =>
 	 (lambda (proc) 
 	   (obituary proc status)
-	   (push-reaped-proc proc)))))
+	   (push-reaped-proc proc)
+	   ))))
 
 
 ;;; All you have to do, if a wait on proc was successful
 (define (waited-by-wait proc status)
-  (obituary proc status)
-  (mark-proc-waited! proc))
+  (if (not (status:stop-sig status))
+      (begin
+	(obituary proc status)
+	(mark-proc-waited! proc))))
 
 ;;; we know from somewhere that proc is dead
 (define (obituary proc status)
@@ -373,13 +400,16 @@
 (define (really-wait-any flags)
   (if (zero? (bitwise-and flags wait/poll))
       (error "real-wait-any without wait/poll" flags))
-    (receive (pid status) 
-       (%wait-any flags)
-       (if pid
-	   (let ((proc (new-child-proc pid)))
-	     (waited-by-wait proc status)
-	     (values proc status))
-	   (values #f #f))))
+  (with-lock
+   wait-lock
+   (lambda ()
+     (receive (pid status)
+        (%wait-any flags)
+	(if pid
+	    (let ((proc (new-child-proc pid)))
+	      (waited-by-wait proc status)
+	      (values proc status))
+	    (values #f #f))))))
       
 	     
 ;;; (wait-process-group [proc-group flags]) => [proc status]
@@ -424,7 +454,10 @@
 
 ;;; (%wait-any flags) (%wait-pid pid flags) (%wait-process-group pgrp flags)
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Direct interfaces to waitpid(2) call.
+;;; Direct interfaces to waitpid(2) call. As opposed to %wait-pid this
+;;; waits on any child (using -1) and gets along if no child is alive
+;;; at all (i.e. catches errno/child).
+
 ;;; [#f #f] means no processes ready on a non-blocking wait.
 ;;; [#f #t] means no waitable process on wait-any.
 
@@ -485,7 +518,7 @@
   (next (make-placeholder))
   prev)
 
-(define reaped-proc-tail (make-placeholder))
+(define reaped-proc-tail (make-reaped-proc (make-weak-pointer #f) 'head))
 (define reaped-proc-head reaped-proc-tail)
 (define reaped-proc-push-lock (make-lock))
 (define reaped-proc-pop-lock (make-lock))          ;;; Zippy sez: pop lock!
@@ -493,22 +526,29 @@
 (define (push-reaped-proc proc)
   (obtain-lock reaped-proc-push-lock)
   (let ((push-me (make-reaped-proc (make-weak-pointer proc) reaped-proc-tail)))
-    (placeholder-set! reaped-proc-tail push-me)
-;    (add-finalizer! proc (lambda ignore (remove-reaped-proc push-me)))
-    (set! reaped-proc-tail (reaped-proc:next push-me)))
+    (placeholder-set! (reaped-proc:next reaped-proc-tail) push-me)
+    (add-finalizer! proc (make-reaped-proc-finalizer push-me))
+    (set! reaped-proc-tail push-me))
   (release-lock reaped-proc-push-lock))
 
+(define (make-reaped-proc-finalizer push-me)
+  (lambda ignore 
+    (remove-reaped-proc push-me)))
+
 (define (remove-reaped-proc reaped-proc)
   (spawn (lambda ()                 ;This is blocking, so should run by itself
-	   (placeholder-set! (reaped-proc:prev reaped-proc)
-			     (placeholder-value
-			      (reaped-proc:next reaped-proc))))
-	 "reaped-proc-removing-thread"))
+	   (set-reaped-proc:prev
+	    (placeholder-value (reaped-proc:next reaped-proc))
+	    (reaped-proc:prev reaped-proc))
+	   (set-reaped-proc:next
+	    (reaped-proc:prev reaped-proc)
+	    (reaped-proc:next reaped-proc)))
+  	 "reaped-proc-removing-thread"))
 
 (define (pop-reaped-proc)
   (obtain-lock reaped-proc-pop-lock)        ;;; pop lock pop lock pop lock!
-  (let ((pop-me (placeholder-value reaped-proc-head)))
-    (set! reaped-proc-head (reaped-proc:next pop-me))
+  (let ((pop-me (placeholder-value (reaped-proc:next reaped-proc-head))))
+    (set! reaped-proc-head pop-me)
     (release-lock reaped-proc-pop-lock)
     (weak-pointer-ref (reaped-proc:proc pop-me))))
 
