@@ -191,7 +191,8 @@
   (let-optionals args ((port  (current-input-port))
 		       (start 0)
 		       (end   (string-length buf)))
-
+    (if (immutable? buf)
+        (error "Immutable buffer argument to %read-delimited!" buf))
     (let ((delims (x->char-set delims)))
       (let lp ((start start) (total 0))
 	(receive (terminator num-read) 
