@@ -10,12 +10,8 @@
 #endif
 #include <unistd.h>
 #include <time.h>
-#include "../scheme48.h"
+#include "../c/scheme48.h"
 
-/* Sux because it's dependent on 32-bitness. */
-#define hi8(i)  (((i)>>24) & 0xff)
-#define lo24(i) ((i) & 0xffffff)
-#define comp8_24(hi, lo) (((hi)<<24) + (lo))
 
 /* Sleep until time hisecs/losecs (return #t),
 ** or until interrupted (return #f).
@@ -28,11 +24,10 @@
 ** and is pretty straightforward.
 */
 
-scheme_value sleep_until(int hisecs, int losecs)
+s48_value sleep_until(s48_value scm_when)
 {
-  time_t when = comp8_24(hisecs, losecs);
   time_t now = time(0);
-  int delta = when - now;
+  int delta = s48_extract_integer(scm_when) - now;
   if( delta > 0 ) {
     fd_set r, w, e;
     struct timeval tv;
@@ -41,8 +36,8 @@ scheme_value sleep_until(int hisecs, int losecs)
     FD_ZERO(&r);
     FD_ZERO(&w);
     FD_ZERO(&e);
-    if( select(0, &r, &w, &e, &tv) ) return SCHFALSE;	/* Lose */
+    if( select(0, &r, &w, &e, &tv) ) return S48_FALSE;	/* Lose */
     }
-  return SCHTRUE; /* Win */
+  return S48_TRUE; /* Win */
   }
   
