@@ -124,7 +124,6 @@
 	  socket/linger
 	  socket/oob-inline
 	  socket/use-privileged
-	  socket/cant-signal
 	  socket/send-buffer
 	  socket/receive-buffer
 	  socket/send-low-water
@@ -133,8 +132,6 @@
 	  socket/receive-timeout
 	  socket/error
 	  socket/type
-	  ip/options
-	  ip/time-to-live
 	  tcp/no-delay
 	  tcp/max-segment))
 
