@@ -15,7 +15,7 @@
 ;;; You can't throw an error within a handler
 ;;;
 
-;;; Move this to somewhere else as soon as Marc published his SRFI
+;;; Move this to somewhere else as soon as Marc has published his SRFI
 (define (continuation-capture receiver)
   ((call-with-current-continuation
     (lambda (cont)
@@ -49,7 +49,7 @@
 			     (continuation-graft
 			      cont
 			      (lambda ()
-				(apply errno-error 
+				(apply errno-error-with-message
 				       (cadr stuff)   ; errno
 				       (caddr stuff)  ;msg
 				       syscall 
@@ -111,22 +111,22 @@
 
 (import-os-error-syscall user-gid  () "scsh_getgid")
 
-(import-os-error-syscall user-effective-gid () "scsh_getegid")
+(import-os-error-syscall process-user-effective-gid () "scsh_getegid")
 
-(import-os-error-syscall set-gid (gid) "scsh_setgid")
+(import-os-error-syscall process-set-gid (gid) "scsh_setgid")
 
-(import-os-error-syscall set-effective-gid (gid) "scsh_setegid")
+(import-os-error-syscall set-process-user-effective-gid (gid) "scsh_setegid")
 
 (import-os-error-syscall user-supplementary-gids () "get_groups")
 
 ;;; UID
 (import-os-error-syscall user-uid  () "scsh_getuid")
 
-(import-os-error-syscall user-effective-uid () "scsh_geteuid")
+(import-os-error-syscall process-user-effective-uid () "scsh_geteuid")
 
-(import-os-error-syscall set-uid (uid) "scsh_setuid")
+(import-os-error-syscall process-set-uid (uid) "scsh_setuid")
 
-(import-os-error-syscall set-effective-uid (uid) "scsh_seteuid")
+(import-os-error-syscall set-process-user-effective-uid (uid) "scsh_seteuid")
 
 (import-os-error-syscall %user-login-name () "my_username")
  
@@ -143,7 +143,7 @@
 
 (import-os-error-syscall process-group () "scsh_getpgrp")
 
-(import-os-error-syscall %set-process-group (pid groupid) "setpgid")
+(import-os-error-syscall %set-process-group (pid groupid) "scsh_setpgid")
 
 (define (set-process-group arg1 . maybe-arg2)
   (receive (pid pgrp) (if (null? maybe-arg2)
@@ -183,7 +183,9 @@
 ;;; ports.
 
 (define (generic-file-op thing fd-op fname-op)
-  (if (string? thing) (fname-op thing)
+  (if (string? thing) 
+      (with-resources-aligned (list cwd-resource euid-resource egid-resource) 
+			      (lambda () (fname-op thing)))
       (call/fdes thing fd-op)))
 
 
@@ -241,11 +243,11 @@
 	(fname (ensure-file-name-is-nondirectory path)))
     (%%create-directory fname mode)))
 
-(import-os-error-syscall read-symlink (path) "scsh_readlink")
+(import-os-error-syscall %read-symlink (path) "scsh_readlink")
 
 (import-os-error-syscall %rename-file (old-name new-name) "scsh_rename")
 
-(import-os-error-syscall delete-directory (path) "scsh_rmdir")
+(import-os-error-syscall %delete-directory (path) "scsh_rmdir")
 
 (import-os-error-syscall %utime (path ac m) "scm_utime") 
 
@@ -254,15 +256,18 @@
 ;;; (SET-FILE-TIMES path [access-time mod-time])
 
 (define (set-file-times path . maybe-times)
-  (if (pair? maybe-times)
-      (let* ((access-time (real->exact-integer (car maybe-times)))
-	     (mod-time (if (pair? (cddr maybe-times))
-			   (error "Too many arguments to set-file-times/errno"
-				  (cons path maybe-times))
-			   (real->exact-integer (cadr maybe-times)))))
-	(%utime path access-time 
-		           mod-time ))
-      (%utime-now path)))
+  (with-resources-aligned 
+   (list cwd-resource euid-resource egid-resource) 
+   (lambda ()      
+     (if (pair? maybe-times)
+	 (let* ((access-time (real->exact-integer (car maybe-times)))
+		(mod-time (if (pair? (cddr maybe-times))
+			      (error "Too many arguments to set-file-times/errno"
+				     (cons path maybe-times))
+			      (real->exact-integer (cadr maybe-times)))))
+	   (%utime path access-time 
+		   mod-time ))
+	 (%utime-now path)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; STAT
@@ -312,9 +317,8 @@
 						      
 		
 (define (file-info fd/port/fname . maybe-chase?)
- (with-cwd-aligned
-   (let ((chase? (:optional maybe-chase? #t)))
-     (%file-info fd/port/fname chase?))))
+  (let ((chase? (:optional maybe-chase? #t)))
+    (%file-info fd/port/fname chase?)))
 
 
 (define file-attributes
@@ -338,7 +342,11 @@
 		   (lambda (fd)    (%truncate-fdes fd    length))
 		   (lambda (fname) (%truncate-file fname length))))
 
-(import-os-error-syscall delete-file (path) "scsh_unlink")
+(import-os-error-syscall %delete-file (path) "scsh_unlink")
+
+(define (delete-file path)
+  (with-resources-aligned (list cwd-resource euid-resource egid-resource) 
+			  (lambda () (%delete-file path))))
 
 (import-os-error-syscall %sync-file (fd) "scsh_fsync")
 
@@ -367,6 +375,12 @@
 (define seek/end 2)
 
 (define (seek fd/port offset . maybe-whence)
+  (if (and (open-input-port? fd/port)
+           (> (byte-vector-length (port-buffer fd/port)) 1))
+      (error "Seek does currently not work on buffered ports" fd/port))
+  (if (and (open-output-port? fd/port)
+           (> (byte-vector-length (port-buffer fd/port)) 0))
+      (error "Seek does currently not work on buffered ports" fd/port))
   (let ((whence (:optional maybe-whence seek/set))
 	(fd (if (integer? fd/port) fd/port (port->fdes fd/port))))
     (%fd-seek fd offset whence)))
@@ -380,8 +394,9 @@
 (import-os-error-syscall %open (path flags mode) "scsh_open")
 
 (define (open-fdes path flags . maybe-mode) ; mode defaults to 0666
-    (with-cwd-aligned
-     (with-umask-aligned
+    (with-resources-aligned 
+     (list cwd-resource umask-resource euid-resource egid-resource)
+     (lambda ()
       (%open path flags (:optional maybe-mode #o666)))))
 
 
@@ -423,9 +438,6 @@
 ;;; (define-errno-syscall (signal-process-group proc-group signal)
 ;;;   signal-process-group/errno)
 
-(define (pause-until-interrupt)
-  (next-sigevent (most-recent-sigevent) full-interrupt-set))
-
 
 ;;; User info
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -527,18 +539,20 @@
 ;;; Directory stuff
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(import-os-error-syscall %open-dir (dir-name) "open_dir")
+(import-os-error-syscall %open-dir (dir-name) "directory_files")
 
 (define (directory-files . args)
-  (with-cwd-aligned
-   (let-optionals args ((dir       ".")
-			(dotfiles? #f))
-    (check-arg string? dir directory-files)
-    (let* ((files (%open-dir (ensure-file-name-is-nondirectory dir)))
-	   (files-sorted ((structure-ref sort sort-list!) files filename<=)))
-      (if dotfiles? files-sorted
-	  (filter (lambda (f) (not (dotfile? f)))
-		  files-sorted))))))
+  (with-resources-aligned
+   (list cwd-resource euid-resource egid-resource)
+   (lambda ()
+     (let-optionals args ((dir       ".")
+			  (dotfiles? #f))
+       (check-arg string? dir directory-files)
+       (let* ((files (%open-dir (ensure-file-name-is-nondirectory dir)))
+	      (files-sorted ((structure-ref sort sort-list!) files filename<=)))
+	 (if dotfiles? files-sorted
+	     (filter (lambda (f) (not (dotfile? f)))
+		     files-sorted)))))))
 
 (define (dotfile? f)
   (char=? (string-ref f 0) #\.))
@@ -552,6 +566,45 @@
 	  #f
 	  (string<= f1 f2))))
 
+; A record for directory streams.  It just has the name and a byte vector
+; containing the C directory object.  The name is used only for printing.
+
+(define-record directory-stream
+  name
+  c-dir)
+
+(define-record-discloser type/directory-stream
+  (lambda (ds)
+    (list 'directory-stream (directory-stream:name ds))))
+
+; Directory streams are meaningless in a resumed image.
+(define-record-resumer type/directory-stream #f)
+
+(define (open-directory-stream name)
+  (let ((dir (make-directory-stream 
+	      name
+	      (with-resources-aligned 
+	       (list cwd-resource euid-resource egid-resource)
+	       (lambda ()
+		 (open-dir name))))))
+    (add-finalizer! dir close-directory-stream)
+    dir))
+
+(define (read-directory-stream dir-stream)
+  (read-dir (directory-stream:c-dir dir-stream)))
+
+(define (close-directory-stream dir-stream)
+  (let ((c-dir (directory-stream:c-dir dir-stream)))
+    (if c-dir
+	(begin
+	  (close-dir c-dir)
+	  (set-directory-stream:c-dir dir-stream #f)))))
+
+(import-os-error-syscall open-dir (name) "scm_opendir")
+(import-os-error-syscall close-dir (dir-stream) "scm_closedir")
+(import-os-error-syscall read-dir (dir-stream) "scm_readdir")
+
+
 ;;; I do this one in C, I'm not sure why:
 ;;; It is used by MATCH-FILES.
 ;;; 99/7: No one is using this function, so I'm commenting it out.
@@ -701,3 +754,14 @@
 	(error "illegal char in salt " salt))
     (if (> (string-length key) 8) (error "key too long "  (string-length key)))
     (%crypt key salt)))
+
+(define-record uname
+  os-name
+  node-name
+  release
+  version
+  machine)
+
+(define-exported-binding "uname-record-type" type/uname)
+
+(import-os-error-syscall uname () "scm_uname")
