@@ -7,12 +7,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/signal.h>
+#include <signal.h>
 #include <sys/types.h>
 #include <sys/times.h>
 #include <sys/time.h>
 #include <fcntl.h>		/* for O_RDWR */
 #include <sys/stat.h>
 #include <sys/param.h> /* For gethostname() */
+#include <sys/utsname.h> /* for uname() */
 
 #include <netdb.h>
 /* This lossage brought to you by Solaris and BIND */
@@ -35,6 +37,7 @@
 #include <crypt.h> 
 #endif
 #include "cstuff.h"
+#include <limits.h>     /* NGROUPS_MAX on solaris */
 
 /* Make sure our exports match up w/the implementation: */
 #include "syscalls1.h"
@@ -58,16 +61,36 @@ extern char **environ;
 s48_value wait_pid(s48_value s48_pid, s48_value s48_flags)
 {
   int status=0;
-  pid_t pid = (pid_t) s48_extract_integer (s48_pid); 
-  int flags = s48_extract_integer (s48_flags); 
+  pid_t pid;
+  int flags;
   pid_t result_pid;
+  s48_value sch_retval = S48_UNSPECIFIC;
+
+  s48_value sch_result_pid = S48_UNSPECIFIC;
+  s48_value sch_status = S48_UNSPECIFIC;
+  s48_value sch_status_list = S48_UNSPECIFIC;
+  S48_DECLARE_GC_PROTECT(5);
+
+  S48_GC_PROTECT_5(s48_pid, s48_flags,
+		   sch_result_pid, sch_status, sch_status_list);
+
+  pid = (pid_t) s48_extract_integer (s48_pid); 
+  flags = s48_extract_integer (s48_flags); 
 
   result_pid = waitpid(pid, &status, flags);
   if (result_pid == -1)
-    s48_raise_os_error_2 (errno, s48_pid, s48_flags);
-  return s48_cons (s48_enter_integer (result_pid),
-		   s48_cons (s48_enter_integer (status),
-			     S48_NULL));
+    if (errno == ECHILD) sch_result_pid = s48_enter_fixnum(0);
+    else s48_raise_os_error_2 (errno, s48_pid, s48_flags);
+  else {
+    sch_result_pid = s48_enter_integer (result_pid);
+    sch_status = s48_enter_integer (status);
+  }
+  sch_status_list = s48_cons (sch_status, S48_NULL);
+  sch_retval = s48_cons (sch_result_pid, sch_status_list);
+
+  S48_GC_UNPROTECT();
+
+  return sch_retval;
 }
 
 
@@ -109,15 +132,16 @@ s48_value scheme_exec(s48_value prog, s48_value argv, s48_value env)
       unix_env[j] = s48_extract_string(S48_VECTOR_REF(env,j));
     unix_env[envlen] = NULL;
   }
-
+  s48_stop_alarm_interrupts();
   execve(s48_extract_string (prog), unix_argv, unix_env); /* Do it. */
   e = errno;
+  s48_start_alarm_interrupts();
   if( env != S48_TRUE ) {
     Free(unix_env);
   }
   Free(unix_argv);
-  return s48_enter_fixnum(e); // Don't raise an exception since
-                              // we're maybe just testing the path
+  return s48_enter_integer(e); /* Don't raise an exception since
+				  we're maybe just testing the path */
 }
 
 s48_value scsh_exit(s48_value status)
@@ -148,12 +172,16 @@ s48_value scsh_fork()
 s48_value scheme_pipe()
 {
   int fds[2];
+  s48_value sch_retval = S48_UNSPECIFIC;
+
   if(pipe(fds) == -1) 
     s48_raise_os_error(errno);
-  else
-    return s48_cons (s48_enter_fixnum (fds[0]),
-		     s48_cons (s48_enter_fixnum (fds [1]),
-			       S48_NULL));
+  else {
+    sch_retval = s48_cons(s48_enter_fixnum(fds[0]),
+			  s48_cons(s48_enter_fixnum (fds[1]), S48_NULL));
+  }
+
+    return sch_retval;
 }
 
 s48_value scsh_kill (s48_value sch_pid, s48_value sch_signal)
@@ -167,7 +195,7 @@ s48_value scsh_kill (s48_value sch_pid, s48_value sch_signal)
   
 
 /* Read the symlink. */
-
+#ifdef MAXPATHLEN
 s48_value scsh_readlink(s48_value sch_path)
 {
   char linkpath[MAXPATHLEN+1];
@@ -180,6 +208,37 @@ s48_value scsh_readlink(s48_value sch_path)
       return s48_enter_string(linkpath);
     }
 }
+#else
+s48_value scsh_readlink(s48_value sch_path)
+{
+  char *linkpath;
+  int size;
+  int retval;
+  s48_value sch_sym_link_path = S48_UNSPECIFIC;
+  
+  for (size = 256;; size *=2){
+
+    linkpath = Malloc(char,size);
+    if (!linkpath)
+      s48_raise_os_error_1(errno, sch_path);
+
+    retval = readlink(s48_extract_string (sch_path), linkpath, size);
+
+    if (retval == -1){
+      free (linkpath);
+      s48_raise_os_error_1(errno, sch_path);
+    }
+    
+    if (retval < size){
+      sch_sym_link_path = s48_enter_substring (linkpath, retval);
+      free (linkpath);
+      return sch_sym_link_path;
+    }
+    free (linkpath);
+  }
+}
+#endif
+
 
 s48_value scsh_rename(s48_value sch_from, s48_value sch_to)
 {
@@ -206,12 +265,19 @@ s48_value scsh_rmdir(s48_value sch_path)
 
 s48_value scm_utime(s48_value sch_path, s48_value sch_ac, s48_value sch_mod)
 {
-    struct utimbuf t;
-    int retval;
-    t.actime = s48_extract_integer (sch_ac);
-    t.modtime = s48_extract_integer (sch_mod);
-    retval = utime(s48_extract_string (sch_path), &t);
-    if (retval == -1)
+  struct utimbuf t;
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_path, sch_ac, sch_mod);
+
+  t.actime = s48_extract_integer (sch_ac);
+  t.modtime = s48_extract_integer (sch_mod);
+  retval = utime(s48_extract_string (sch_path), &t);
+
+  S48_GC_UNPROTECT();
+
+  if (retval == -1)
     s48_raise_os_error_3(errno, sch_path, sch_ac, sch_mod);
   return S48_UNSPECIFIC;
 }
@@ -269,6 +335,7 @@ s48_value scheme_cwd()
 {
   char *buf;
   int size = scsh_path_max + 1; /* +1 for terminating nul byte... */
+  s48_value cwd = S48_UNSPECIFIC;
 
   buf = Malloc(char,size);
   if(!buf) goto lose;
@@ -281,8 +348,11 @@ s48_value scheme_cwd()
       if( !nbuf ) goto lose;
       buf = nbuf;
     }
-
-  return s48_enter_string(buf);		/* win */
+  cwd = s48_enter_string(buf);		/* win */
+  
+  Free(buf);
+  
+  return cwd;
 
  lose:
   {int e = errno;
@@ -301,14 +371,37 @@ s48_value scheme_cwd()
 s48_value process_times()
 {
   struct tms tms;
+
+  s48_value sch_result_utime = S48_UNSPECIFIC;
+  s48_value sch_result_stime = S48_UNSPECIFIC;
+  s48_value sch_result_cutime = S48_UNSPECIFIC;
+  s48_value sch_result_cstime = S48_UNSPECIFIC;
+  s48_value sch_result_retval = S48_UNSPECIFIC;
+
+  S48_DECLARE_GC_PROTECT(4);
+
   clock_t t = times(&tms);
   if (t == -1) s48_raise_os_error(errno);
-  return 
-    s48_cons(s48_enter_integer (tms.tms_utime),
-	     s48_cons(s48_enter_integer (tms.tms_stime),
-		      s48_cons(s48_enter_integer (tms.tms_cutime),
-			       s48_cons(s48_enter_integer (tms.tms_cstime),
-					S48_NULL))));
+
+  S48_GC_PROTECT_4(sch_result_utime,
+		   sch_result_stime,
+		   sch_result_cutime,
+		   sch_result_cstime);
+		  
+  sch_result_utime = s48_enter_integer(tms.tms_utime);
+  sch_result_stime = s48_enter_integer(tms.tms_stime);
+  sch_result_cutime = s48_enter_integer(tms.tms_cutime);
+  sch_result_cstime = s48_enter_integer(tms.tms_cstime);
+
+
+  sch_result_retval = s48_cons(sch_result_utime,
+			       s48_cons(sch_result_stime,
+					s48_cons(sch_result_cutime,
+						 s48_cons(sch_result_cstime, S48_NULL))));
+
+  S48_GC_UNPROTECT();
+
+  return sch_result_retval;
 }
 
 s48_value cpu_clock_ticks_per_sec() 
@@ -333,8 +426,16 @@ s48_value cpu_clock_ticks_per_sec()
 
 s48_value scsh_chmod(s48_value sch_path, s48_value sch_mode)
 {
-  int retval = chmod (s48_extract_string(sch_path),
-		      s48_extract_integer(sch_mode));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_path, sch_mode);
+
+  retval = chmod (s48_extract_string(sch_path),
+		  s48_extract_integer(sch_mode));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_path, sch_mode);
   return S48_UNSPECIFIC;
@@ -342,8 +443,16 @@ s48_value scsh_chmod(s48_value sch_path, s48_value sch_mode)
 
 s48_value scsh_fchmod(s48_value sch_fd, s48_value sch_mode)
 {
-  int retval = fchmod (s48_extract_fixnum(sch_fd),
-		       s48_extract_integer(sch_mode));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_fd, sch_mode);
+
+  retval = fchmod (s48_extract_fixnum(sch_fd),
+		   s48_extract_integer(sch_mode));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_fd, sch_mode);
   return S48_UNSPECIFIC;
@@ -351,10 +460,17 @@ s48_value scsh_fchmod(s48_value sch_fd, s48_value sch_mode)
 
 s48_value scsh_chown(s48_value sch_path, s48_value sch_uid, s48_value sch_gid)
 {
-  int retval = chown(s48_extract_string(sch_path),
-		     s48_extract_integer(sch_uid),
-		     s48_extract_integer(sch_gid));
-  
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_path, sch_uid, sch_gid);
+
+  retval = chown(s48_extract_string(sch_path),
+		 s48_extract_integer(sch_uid),
+		 s48_extract_integer(sch_gid));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_3(errno, sch_path, sch_uid, sch_gid);
   return S48_UNSPECIFIC;
@@ -362,9 +478,16 @@ s48_value scsh_chown(s48_value sch_path, s48_value sch_uid, s48_value sch_gid)
 
 s48_value scsh_fchown(s48_value sch_fd, s48_value sch_uid, s48_value sch_gid)
 {
-  int retval = fchown(s48_extract_fixnum(sch_fd),
-		      s48_extract_integer(sch_uid),
-		      s48_extract_integer(sch_gid));
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_fd, sch_uid, sch_gid);
+
+  retval = fchown(s48_extract_fixnum(sch_fd),
+		  s48_extract_integer(sch_uid),
+		  s48_extract_integer(sch_gid));
+
+  S48_GC_UNPROTECT();
   
   if (retval == -1)
     s48_raise_os_error_3(errno, sch_fd, sch_uid, sch_gid);
@@ -373,8 +496,16 @@ s48_value scsh_fchown(s48_value sch_fd, s48_value sch_uid, s48_value sch_gid)
 
 s48_value scsh_access(s48_value sch_path, s48_value sch_mode)
 {
-  int retval = access (s48_extract_string(sch_path),
-		       s48_extract_integer(sch_mode));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_path, sch_mode);
+
+  retval = access (s48_extract_string(sch_path),
+		   s48_extract_integer(sch_mode));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_path, sch_mode);
   return S48_UNSPECIFIC;
@@ -489,8 +620,16 @@ s48_value scsh_symlink(s48_value sch_name1, s48_value sch_name2)
 
 s48_value scsh_truncate(s48_value sch_path, s48_value sch_length)
 {
-  int retval = truncate (s48_extract_string (sch_path),
-			 s48_extract_integer (sch_length));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_path, sch_length);
+
+  retval = truncate (s48_extract_string (sch_path),
+		     s48_extract_integer (sch_length));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_path, sch_length);
   return S48_UNSPECIFIC;
@@ -498,8 +637,17 @@ s48_value scsh_truncate(s48_value sch_path, s48_value sch_length)
 
 s48_value scsh_ftruncate(s48_value sch_fdes, s48_value sch_length)
 {
-  int retval = ftruncate (s48_extract_fixnum (sch_fdes),
-			  s48_extract_integer (sch_length));
+  int retval;
+
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_fdes, sch_length);
+
+  retval = ftruncate (s48_extract_fixnum (sch_fdes),
+		      s48_extract_integer (sch_length));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_fdes, sch_length);
   return S48_UNSPECIFIC;
@@ -523,7 +671,9 @@ s48_value scsh_fsync(s48_value sch_fdes)
 
 s48_value scsh_sync()
 {
+#ifdef HAVE_SYNC
   sync();
+#endif
   return S48_UNSPECIFIC;
 }
 
@@ -557,9 +707,17 @@ s48_value scsh_dup2(s48_value sch_oldd,  s48_value sch_newd)
 s48_value scsh_lseek(s48_value sch_fdes, s48_value sch_offset, 
 		     s48_value sch_whence)
 {
-  int retval = lseek (s48_extract_fixnum (sch_fdes),
-		      s48_extract_integer (sch_offset),
-		      s48_extract_fixnum (sch_whence));
+  int retval;
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(sch_fdes, sch_offset, sch_whence);
+
+  retval = lseek (s48_extract_fixnum (sch_fdes),
+		  s48_extract_integer (sch_offset),
+		  s48_extract_fixnum (sch_whence));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_3 (errno, sch_fdes, sch_offset, sch_whence);
   return s48_enter_integer (retval);
@@ -614,7 +772,15 @@ s48_value scsh_getegid()
 
 s48_value scsh_setgid(s48_value gid)
 {
-  int retval = setgid (s48_extract_integer (gid));
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(gid);
+
+  retval = setgid (s48_extract_integer (gid));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, gid);
   return S48_UNSPECIFIC;
@@ -622,11 +788,19 @@ s48_value scsh_setgid(s48_value gid)
 
 s48_value scsh_setegid(s48_value gid)
 {
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(gid);
+
 #ifdef HAVE_SETEGID
-  int retval = setegid (s48_extract_integer (gid));
+  retval = setegid (s48_extract_integer (gid));
 #else 
-  int retval = setregid (-1, s48_extract_integer (gid));
+  retval = setregid (-1, s48_extract_integer (gid));
 #endif
+  
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, gid);
   return S48_UNSPECIFIC;
@@ -637,30 +811,22 @@ s48_value scsh_setegid(s48_value gid)
 s48_value get_groups()
 {
   int retval;
-  int veclen = getgroups(0,NULL);
+  gid_t gvec[NGROUPS_MAX + 1];
+  int veclen = getgroups(NGROUPS_MAX + 1, gvec);
   int i = veclen;
   s48_value l = S48_NULL;
-  gid_t gvec0[20], *gp = gvec0;
 
-  if( veclen > 20 )
-    if (NULL == (gp=Malloc(gid_t,veclen))) s48_raise_os_error(errno);
-
-  retval = getgroups(veclen, gp);
-
-  if (retval == -1){
-    if (veclen > 20) Free(gp);
+  if (veclen == -1)
     s48_raise_os_error(errno);
-  }
-  else if (retval != veclen){     // paranoia...
-    get_groups();
-  }
-  else{
-    while (i > 0){
-      l = s48_cons(s48_enter_integer(gp[--i]), l);
-    }
-    
-    if (veclen > 20) Free(gp);
+  else {
+    S48_DECLARE_GC_PROTECT(1);
+    S48_GC_PROTECT_1(l);
+
+    while (i > 0)
+      l = s48_cons(s48_enter_integer(gvec[--i]), l);
     
+    S48_GC_UNPROTECT();
+
     return l;
   }
 }
@@ -676,7 +842,15 @@ s48_value scsh_geteuid()
 
 s48_value scsh_setuid(s48_value uid)
 {
-  int retval = setuid (s48_extract_integer (uid));
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(uid);
+
+  retval = setuid (s48_extract_integer (uid));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, uid);
   return S48_UNSPECIFIC;
@@ -684,11 +858,19 @@ s48_value scsh_setuid(s48_value uid)
 
 s48_value scsh_seteuid(s48_value uid)
 {
+  int retval;
+  S48_DECLARE_GC_PROTECT(1);
+
+  S48_GC_PROTECT_1(uid);
+
 #ifdef HAVE_SETEUID
-  int retval = seteuid (s48_extract_integer (uid));
+  retval = seteuid (s48_extract_integer (uid));
 #else
-  int retval = setreuid (-1, s48_extract_integer (uid));
+  retval = setreuid (-1, s48_extract_integer (uid));
 #endif
+  
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_1(errno, uid);
   return S48_UNSPECIFIC;
@@ -711,8 +893,16 @@ s48_value scsh_getpgrp()
 
 s48_value scsh_setpgid(s48_value sch_pid, s48_value sch_pgrp)
 {
-  int retval = setpgid(s48_extract_integer(sch_pid),
-		       s48_extract_integer(sch_pgrp));
+  int retval;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(sch_pid, sch_pgrp);
+
+  retval = setpgid(s48_extract_integer(sch_pid),
+		   s48_extract_integer(sch_pgrp));
+
+  S48_GC_UNPROTECT();
+
   if (retval == -1)
     s48_raise_os_error_2(errno, sch_pid, sch_pgrp);
   return S48_UNSPECIFIC;
@@ -881,8 +1071,16 @@ s48_value errno_msg(s48_value sch_i)
 
 s48_value fcntl_read(s48_value fd, s48_value command)
 { 
-  int ret = fcntl(s48_extract_fixnum (fd), 
-				 s48_extract_integer (command)); 
+  int ret;
+  S48_DECLARE_GC_PROTECT(2);
+
+  S48_GC_PROTECT_2(fd, command);
+
+  ret = fcntl(s48_extract_fixnum (fd), 
+	      s48_extract_integer (command)); 
+
+  S48_GC_UNPROTECT();
+
   if (ret == -1)
     s48_raise_os_error_2(errno, fd, command);
   else return s48_enter_fixnum (ret); 
@@ -891,9 +1089,18 @@ s48_value fcntl_read(s48_value fd, s48_value command)
 
 s48_value fcntl_write(s48_value fd, s48_value command, s48_value value)
 { 
-  int ret = fcntl(s48_extract_fixnum (fd), 
-		  s48_extract_integer (command), 
-		  s48_extract_integer (value));
+  int ret;
+
+  S48_DECLARE_GC_PROTECT(3);
+
+  S48_GC_PROTECT_3(fd, command, value);
+
+  ret = fcntl(s48_extract_fixnum (fd), 
+	      s48_extract_integer (command), 
+	      s48_extract_integer (value));
+
+  S48_GC_UNPROTECT();
+  
   if (ret == -1)
     s48_raise_os_error_3(errno, fd, command, value);
   else return s48_enter_fixnum (ret); 
@@ -907,12 +1114,34 @@ s48_value scm_crypt(s48_value key, s48_value salt)
   char * ret = crypt (s48_extract_string (key), 
 		      s48_extract_string(salt));
 
-  // FreeBSD does this on error:
+  /* FreeBSD does this on error:*/
   if (ret == NULL) return s48_enter_string("");
 
   return s48_enter_string (ret);
 }
 
+static s48_value uname_record_type_binding = S48_FALSE;
+
+s48_value scm_uname(void)
+{
+  s48_value uname_record = S48_UNSPECIFIC;
+  struct utsname uname_struct;
+  S48_DECLARE_GC_PROTECT(1);
+  
+  S48_GC_PROTECT_1(uname_record);
+  if (uname(&uname_struct) == -1)
+    s48_raise_os_error(errno);
+  
+  uname_record = s48_make_record(uname_record_type_binding);
+  S48_RECORD_SET(uname_record, 0, s48_enter_string (uname_struct.sysname));
+  S48_RECORD_SET(uname_record, 1, s48_enter_string (uname_struct.nodename));
+  S48_RECORD_SET(uname_record, 2, s48_enter_string (uname_struct.release));
+  S48_RECORD_SET(uname_record, 3, s48_enter_string (uname_struct.version));
+  S48_RECORD_SET(uname_record, 4, s48_enter_string (uname_struct.machine));
+  S48_GC_UNPROTECT();
+  return uname_record;
+}
+
 void s48_init_syscalls (){
   S48_EXPORT_FUNCTION(scheme_exec);
   S48_EXPORT_FUNCTION(scsh_exit);
@@ -933,7 +1162,7 @@ void s48_init_syscalls (){
   S48_EXPORT_FUNCTION(scsh_getpid);
   S48_EXPORT_FUNCTION(scsh_getppid);
   S48_EXPORT_FUNCTION(scsh_getpgrp);
-  S48_EXPORT_FUNCTION(setpgid);
+  S48_EXPORT_FUNCTION(scsh_setpgid);
   S48_EXPORT_FUNCTION(scsh_setsid);
   S48_EXPORT_FUNCTION(scsh_umask);
   S48_EXPORT_FUNCTION(process_times);
@@ -978,10 +1207,13 @@ void s48_init_syscalls (){
   S48_EXPORT_FUNCTION(scm_gethostname);
   S48_EXPORT_FUNCTION(errno_msg);
   S48_EXPORT_FUNCTION(scm_crypt);
+  S48_EXPORT_FUNCTION(scm_uname);
   S48_GC_PROTECT_GLOBAL(envvec_record_type_binding);
   S48_GC_PROTECT_GLOBAL(add_envvec_finalizerB_binding);
   S48_GC_PROTECT_GLOBAL(current_env);
+  S48_GC_PROTECT_GLOBAL(uname_record_type_binding);
   envvec_record_type_binding = s48_get_imported_binding("envvec-record-type");
+  uname_record_type_binding = s48_get_imported_binding("uname-record-type");
   add_envvec_finalizerB_binding = 
     s48_get_imported_binding("add-envvec-finalizer!");
 }
