@@ -113,15 +113,19 @@
   (if (pair? args)
       (if (null? (cdr args))
 	  (let ((date (check-arg date? (car args) time)))
-	    (%date->time (date:seconds   date)
-			 (date:minute    date)
-			 (date:hour      date)
-			 (date:month-day date)
-			 (date:month     date)
-			 (date:year      date)
-			 (date:tz-name   date) ; #f or string
-			 (date:tz-secs   date) ; #f or int
-			 (date:summer?   date)))
+	    (let ((err?.time
+		   (%date->time (date:seconds   date)
+				(date:minute    date)
+				(date:hour      date)
+				(date:month-day date)
+				(date:month     date)
+				(date:year      date)
+				(date:tz-name   date) ; #f or string
+				(date:tz-secs   date) ; #f or int
+				(date:summer?   date))))
+	      (if (car err?.time)
+		  (error "Error converting date to time." args)
+		  (cdr err?.time))))
 	  (error "Too many arguments to TIME procedure" args))
       (%time)))	; Fast path for (time).
 
