@@ -5,11 +5,17 @@
 (define make-weak-table make-table)
 
 (define (weak-table-set! table number set-me)
-  (table-set! table number (make-weak-pointer set-me)))
+  (table-set! table number (if set-me (make-weak-pointer set-me) #f)))
 
 (define (weak-table-ref table number)
   (let ((ref (table-ref table number)))
-    (if (weak-pointer? ref) (weak-pointer-ref ref) ref)))
+    (if (weak-pointer? ref)
+	(let ((val (weak-pointer-ref ref)))
+	  (if val
+	      val
+	      (begin (table-set! table number #f)
+		     #f)))
+	  ref)))
 
 (define (weak-table-walk proc table)
   (table-walk
